@@ -22,7 +22,7 @@
   TODO:
 
    * maybe important, requires deeper code changes:
-   - download another file with DHCP/WLAN client counters
+   - download another file containing the DHCP/WLAN client counters
      - better get them merged into the same file
      - download/processing function evolved from synchronous
        call, not suited for multiple URLs
@@ -33,14 +33,14 @@
      - which parts are freed automatically, which are not?
      - is g_unref() the right tool?
    - check return code from g_timeout_add_seconds/g_signal_connect
-     - right now, we wouldn't know if start_timer()/install_handler() failed
+     - for now, we wouldn't know if start_timer()/install_handler() failed
      - what to do in case of error?
    - does g_source_remove destroy the source or does it leak?
      - what data structures does a source use?
 
    * could be useful, requires more thinking:
    - aggregate calls to force_redraw()
-     - right now, we call force_redraw() on any possible change
+     - for now, we call force_redraw() on any possible change
    - skip draw if not on active desktop
      - there's an event for that
      - don't skip update or banner
@@ -100,7 +100,7 @@ HD_DEFINE_PLUGIN_MODULE (CCCHHDoorIsDesktopWidget, ccchh_dooris_desktop_widget,
 
 
 
-// #define DEBUG
+#define DEBUG
 
 #ifdef DEBUG
 
@@ -140,7 +140,7 @@ start_update_timer(CCCHHDoorIsDesktopWidget *self)
 			g_warning(WIDGET_NAME": self->update_interval > UPDATE_INTERVAL_MAX, maybe a bug?");
 			self->update_interval = UPDATE_INTERVAL_MAX;
 		}
-		self->timeout_handler = g_timeout_add_seconds (self->update_interval*60, ccchh_dooris_desktop_widget_on_timeout, self);
+		self->timeout_handler = g_timeout_add_seconds(self->update_interval*60, ccchh_dooris_desktop_widget_on_timeout, self);
 		return TRUE;
 	}
 	return FALSE;
@@ -178,11 +178,11 @@ install_connection_handler(CCCHHDoorIsDesktopWidget *self)
 	if (!self->connection_handler) {
 		// set connected to FALSE, initial event may set it to TRUE
 		self->connected = FALSE;
-		/* Connect signal to receive connection events */
-		self->connection_handler = g_signal_connect (self->connection, "connection-event",
+		// Connect signal to receive connection events
+		self->connection_handler = g_signal_connect(self->connection, "connection-event",
 			G_CALLBACK (ccchh_dooris_desktop_widget_connection_event), self);
-		/* Set automatic events */
-		g_object_set (self->connection, "automatic-connection-events", TRUE, NULL);
+		// Enable automatic events
+		g_object_set(self->connection, "automatic-connection-events", TRUE, NULL);
 		g_debug(WIDGET_NAME": connection monitoring enabled");
 		return TRUE;
 	}
@@ -200,8 +200,8 @@ remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
 
 	if (self->connection_handler) {
-		g_object_set (self->connection, "automatic-connection-events", FALSE, NULL);
-		g_signal_handler_disconnect (self->connection, self->connection_handler);
+		g_object_set(self->connection, "automatic-connection-events", FALSE, NULL);
+		g_signal_handler_disconnect(self->connection, self->connection_handler);
 		self->connection_handler = 0;
 		g_debug(WIDGET_NAME": connection monitoring disabled");
 		return TRUE;
@@ -215,6 +215,8 @@ remove_connection_handler(CCCHHDoorIsDesktopWidget *self)
 
 // load settings from gconf
 // TODO: use gconf_client_get_without_default()?
+//       initial load could return FALSE for booleans
+//       if keys are not set yet
 static void
 load_settings(CCCHHDoorIsDesktopWidget *self)
 {
@@ -449,17 +451,17 @@ draw (CCCHHDoorIsDesktopWidget *self, cairo_t *cr)
 		cairo_surface_destroy(image_overlay);
 	}
 
-	// if button pressed, hightlight with white mask
+	// if button pressed, highlight with white mask
 	if (self->highlighted) {
 		cairo_set_source_rgba(cr, IMAGE_COLOR_HIGHLIGHT, IMAGE_ALPHA_HIGHLIGHT);
 		cairo_mask_surface(cr, self->image_current, 5, 5);
 	}
 
-// debug
-if (!self->timeout_handler) {
-	cairo_set_source_rgba(cr, 0.15, 0.15, 0.15, 0.47);
-	cairo_mask_surface(cr, self->image_current, 5, 5);
-}
+	// dim icon if timer stopped
+	if (!self->timeout_handler) {
+		cairo_set_source_rgba(cr, IMAGE_COLOR_DIM, IMAGE_ALPHA_DIM);
+		cairo_mask_surface(cr, self->image_current, 5, 5);
+	}
 
 	cairo_restore(cr);
 }
@@ -472,6 +474,7 @@ show_status_banner (CCCHHDoorIsDesktopWidget *self)
 
 	time_t timenow, timediff;
 	gchar *text, *ts_text;
+	const gchar *state_text;
 
 	g_debug(WIDGET_NAME": show_status_banner()");
 
@@ -484,37 +487,45 @@ show_status_banner (CCCHHDoorIsDesktopWidget *self)
 		timediff = timenow - self->last_change_ts;
 
 		if (timediff < 60) {
-			ts_text = g_strdup_printf(_("(%lds ago)"), (ulong)timediff);
+			ts_text = g_strdup_printf(_("(%lds ago)"),
+				(ulong)timediff);
 		} else if (timediff < 3600) {
-			ts_text = g_strdup_printf(_("(%ldm %lds ago)"), (ulong)timediff / 60, (ulong)timediff % 60);
+			ts_text = g_strdup_printf(_("(%ldm %lds ago)"),
+				(ulong)timediff / 60,
+				(ulong)timediff % 60);
+		} else if (timediff < 86400) {
+			ts_text = g_strdup_printf(_("(%ldh %ldm %lds ago)"),
+				(ulong)timediff / 3600,
+				(ulong)(timediff % 3600) / 60,
+				(ulong)timediff % 60);
 		} else {
-			ts_text = g_strdup_printf(_("(%ldh %ldm %lds ago)"), (ulong)timediff / 3600, (ulong)(timediff % 3600) / 60, (ulong)timediff % 60);
+			ts_text = g_strdup_printf(_("(%ldd %ldh %ldm %lds ago)"),
+				(ulong)timediff / 86400,
+				(ulong)(timediff % 86400) / 3600,
+				(ulong)(timediff % 3600) / 60,
+				(ulong)timediff % 60);
 		}
 	} else {
 		ts_text = g_strdup("");
 	}
 
-	// assemble string to display
 	switch (self->current_state) {
 		case DOORIS_OPEN:
-			text = g_strdup_printf(("%s %s %s"),
-				_("CCCHH: Door <b>unlocked</b>"), ts_text,
-				(self->connected) ? ("") : _("(offline)"));
-				break;
-//				(self->connected) ? _("(online)") : _("(offline)"));
+			state_text = _("CCCHH: Door <b>unlocked</b>"); break;
 		case DOORIS_CLOSED:
-			text = g_strdup_printf(("%s %s %s"),
-				_("CCCHH: Door locked"), ts_text,
-				(self->connected) ? ("") : _("(offline)"));
-				break;
+			state_text = _("CCCHH: Door locked"); break;
 		default:
-			text = g_strdup_printf(("%s %s %s"),
-				_("CCCHH: Unknown"),
-				(self->connected) ? ("") : _("(offline)"),
-				(self->too_old) ? _("(data too old)") : (""));
-				break;
+			state_text = _("CCCHH: Unknown");
+			// repurpose ts_text to display self->too_old
+			g_free (ts_text);
+			ts_text = gstrdup((self->too_old) ? _("(data too old)") : (""));
+			break;
 	}
 
+	// assemble string to display
+	text = g_strdup_printf("%s %s %s", state_text, ts_text,
+			(self->connected) ? ("") : _("(offline)"));
+
 	// display banner on screen
 	hildon_banner_show_information_with_markup (GTK_WIDGET (self),
 		NULL, text);
@@ -531,13 +542,13 @@ force_redraw (CCCHHDoorIsDesktopWidget *self)
 
 	g_debug(WIDGET_NAME": force_redraw()");
 
-	/* Invalidate the drawing: */
+	// Invalidate the drawing
 	GtkWidget *widget = GTK_WIDGET (self);
 
 	if (!widget->window)
 		return;
 
-	/* Redraw the cairo canvas completely by exposing it: */
+	// Redraw the cairo canvas completely by exposing it
 	GdkRegion *region = gdk_drawable_get_clip_region (widget->window);
 	if (!region)
 		return;
@@ -564,7 +575,7 @@ fetch_update_lock (CCCHHDoorIsDesktopWidget *self)
 		return FALSE;
 	}
 
-	// acknowledge button, make icon transparent
+	// indicate update in progress, make icon transparent
 	self->current_alpha = IMAGE_ALPHA_PROGRESS;
 	force_redraw(self);
 
@@ -578,6 +589,7 @@ fetch_update_unlock (CCCHHDoorIsDesktopWidget *self)
 {
 	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
 
+	// reset icon transparency to normal
 	self->current_alpha = IMAGE_ALPHA_NORMAL;
 	force_redraw(self);
 
@@ -593,12 +605,14 @@ fetch_update_start (CCCHHDoorIsDesktopWidget *self)
 
 	g_debug(WIDGET_NAME": fetch_update_start()");
 
-	// TODO: rethink conn_wlan_only logic here
-	//       for now, update anyway if force_banner is set,
-	//       indicating a button press, even if !connected.
-	//       self->connected does not always represent real
-	//       connection state if conn_wlan_only is set
-	if (!self->connected && !(self->conn_wlan_only && self->force_banner)) {
+	// conn_wlan_only: update anyway if force_banner is set,
+	// indicating a button press, even if !connected.
+	// self->connected does not always represent real
+	// connection state if conn_wlan_only is set.
+	// connected_bearer should be NULL if network is down
+	if (!self->connected &&
+			!(self->conn_wlan_only && self->force_banner &&
+			  self->connected_bearer)) {
 		g_debug(WIDGET_NAME": not connected, not fetching new state");
 		return;
 	}
@@ -610,8 +624,11 @@ fetch_update_start (CCCHHDoorIsDesktopWidget *self)
 
 	g_debug(WIDGET_NAME": begin update from URL: %s", self->update_url);
 
-	self->gvfs_timeout_handler = g_timeout_add_seconds (TIMEOUT_CONNECT, fetch_update_cancel_cb, self);
+	// abort update if timer expires
+	self->gvfs_timeout_handler = g_timeout_add_seconds(TIMEOUT_CONNECT, fetch_update_cancel_cb, self);
 
+	// begin update
+	// next callback: either fetch_update_async_open_cb() or fetch_update_cancel_cb()
 	gnome_vfs_async_open(&self->gvfs_handle, self->update_url, GNOME_VFS_OPEN_READ,
 		GNOME_VFS_PRIORITY_MIN, fetch_update_async_open_cb, self);
 }
@@ -626,13 +643,15 @@ fetch_update_cancel_cb (gpointer data)
 
 	if (g_source_is_destroyed(g_main_current_source())) {
 		// this shouldn't happen
-		g_debug(WIDGET_NAME": update_cancel: source destroyed");
+		g_warning(WIDGET_NAME": update_cancel: source destroyed");
 		return FALSE;
 	}
 
-	g_warning(WIDGET_NAME": I/O timeout, cancelling operation");
+	// cancel gvfs operation
+	g_warning(WIDGET_NAME": I/O timeout, cancelling update");
 	gnome_vfs_async_cancel(self->gvfs_handle);
 
+	// set internal state to unknown and unlock mutex
 	fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
 
 	return FALSE;
@@ -668,9 +687,9 @@ fetch_update_async_open_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	// initialize input buffer with zeroes, so strtok won't read beyond its end
 	memset(self->gvfs_input_buffer, 0, MAX_HTTP_SIZE+1);
 
-	self->gvfs_timeout_handler = g_timeout_add_seconds (TIMEOUT_READ, fetch_update_cancel_cb, self);
+	self->gvfs_timeout_handler = g_timeout_add_seconds(TIMEOUT_READ, fetch_update_cancel_cb, self);
 
-	gnome_vfs_async_read (handle, self->gvfs_input_buffer, MAX_HTTP_SIZE,
+	gnome_vfs_async_read(handle, self->gvfs_input_buffer, MAX_HTTP_SIZE,
 		fetch_update_async_read_cb, self);
 }
 
@@ -698,11 +717,26 @@ fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 
 	g_debug(WIDGET_NAME": read %"GNOME_VFS_SIZE_FORMAT_STR" bytes", bytes_read);
 
-// calling fetch_unlock in cancel_cb() would collide with process()
-//	self->gvfs_timeout_handler = g_timeout_add_seconds (TIMEOUT_CLOSE, fetch_update_cancel_cb, self);
+	if (!self->gvfs_input_buffer) {
+		g_warning(WIDGET_NAME": no input buffer after read");
+		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
+		return;
+	}
+
+	// ensure buffer has a zero byte at MAX_HTTP_SIZE for strtok
+	if (!(self->gvfs_input_buffer = g_try_realloc(self->gvfs_input_buffer, MAX_HTTP_SIZE+1))) {
+		g_warning(WIDGET_NAME": failed to allocate input buffer");
+		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
+		return;
+	}
+	self->gvfs_input_buffer[MAX_HTTP_SIZE] = '\x0';
+
+	// TODO: no timeout on close for now,
+	// cancel_cb would call fetch_update_process() again
+	//self->gvfs_timeout_handler = g_timeout_add_seconds(TIMEOUT_CLOSE, fetch_update_cancel_cb, self);
 
 	// close file
-	gnome_vfs_async_close (handle, fetch_update_async_close_cb, self);
+	gnome_vfs_async_close(handle, fetch_update_async_close_cb, self);
 
 	// meanwhile, parse input buffer
 
@@ -715,23 +749,25 @@ fetch_update_async_read_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	new_report_ts = 0L;
 	tokctx = NULL;
 
+	// fixed-format text response
+
 	// first line of body is either "locked" or "unlocked"
 
-	if (!(line = strtok_r(buffer, "\n", &tokctx))) {
+	if (!(line = strtok_r(self->gvfs_input_buffer, "\n", &tokctx))) {
 		g_warning(WIDGET_NAME": first line of response not found");
-		fetch_update_process(self, new_state, new_change_ts, new_report_ts);
+		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
 		return;
 	}
 
 	if (!strcmp(line, "locked")) {
 		new_state = DOORIS_CLOSED;
-		g_debug(WIDGET_NAME": current state: closed");
+		g_debug(WIDGET_NAME": new state: closed");
 	} else if (!strcmp(line, "unlocked")) {
 		new_state = DOORIS_OPEN;
-		g_debug(WIDGET_NAME": current state: open");
+		g_debug(WIDGET_NAME": new state: open");
 	} else {
-		g_warning(WIDGET_NAME": could not parse response");
-		fetch_update_process(self, new_state, new_change_ts, new_report_ts);
+		g_warning(WIDGET_NAME": could not parse first line of response");
+		fetch_update_process(self, DOORIS_FNORD, 0L, 0L);
 		return;
 	}
 
@@ -822,6 +858,8 @@ fetch_update_process (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
 		return;
 	}
 
+	// state has changed since last update
+
 	// release old icon surface
 	if (self->image_current) {
 		cairo_surface_destroy(self->image_current);
@@ -875,6 +913,7 @@ fetch_update_process (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
 
 // event handlers
 
+// timer event - trigger update
 static gboolean
 ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 {
@@ -884,7 +923,7 @@ ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 
 	if (g_source_is_destroyed(g_main_current_source())) {
 		// this shouldn't happen
-		g_debug(WIDGET_NAME": on_timeout: source destroyed");
+		g_warning(WIDGET_NAME": on_timeout: source destroyed");
 		return FALSE;
 	}
 
@@ -892,9 +931,10 @@ ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 	// connection event starts it again
 	if (!self->connected) {
 		g_debug(WIDGET_NAME": not connected, stopping update timer");
-		// TODO: is this safe? how about remove_source from within the event?
+		// TODO: do we need remove_source() here?
 		self->timeout_handler = 0;
 		force_redraw(self);
+
 		return FALSE;
 	}
 
@@ -904,6 +944,7 @@ ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 }
 
 
+// expose event - draw icon
 static gboolean
 ccchh_dooris_desktop_widget_expose_event (GtkWidget *widget, GdkEventExpose *event)
 {    
@@ -937,7 +978,7 @@ ccchh_dooris_desktop_widget_expose_event (GtkWidget *widget, GdkEventExpose *eve
 }
 
 
-// handle the button press event, highlight icon
+// button press event - highlight icon
 static gboolean 
 ccchh_dooris_desktop_widget_on_button_press_event (GtkWidget *widget, 
   GdkEventButton * event G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
@@ -946,7 +987,7 @@ ccchh_dooris_desktop_widget_on_button_press_event (GtkWidget *widget,
 
 	g_debug(WIDGET_NAME": ccchh_dooris_desktop_widget_on_button_press_event()");
  
-	// TODO: do we really need a button lock? fetch_update locks itself now
+	// TODO: do we really need a button lock? fetch_update locks itself
 	if (!g_static_mutex_trylock(&(self->button_lock))) {
 		g_debug(WIDGET_NAME": button_lock set, not updating");
 		return TRUE;
@@ -959,8 +1000,9 @@ ccchh_dooris_desktop_widget_on_button_press_event (GtkWidget *widget,
 }
 
 
-// handle the leave notify event, remove highlight
-// TODO: should return immediately if not locked, i.e. no previous button-press
+// button leave notify event - remove highlight
+// TODO: should check mutex and return immediately if not locked,
+//       i.e. no previous button-press
 static gboolean 
 ccchh_dooris_desktop_widget_on_button_leave_event (GtkWidget *widget, 
   GdkEventButton * event G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
@@ -978,7 +1020,7 @@ ccchh_dooris_desktop_widget_on_button_leave_event (GtkWidget *widget,
 }
 
 
-// handle the button release event, begin update, remove highlight
+// button release event - begin update, remove highlight
 static gboolean 
 ccchh_dooris_desktop_widget_on_button_release_event (GtkWidget *widget, 
   GdkEventButton * event G_GNUC_UNUSED, gpointer user_data G_GNUC_UNUSED)
@@ -1019,6 +1061,7 @@ ccchh_dooris_desktop_widget_on_button_release_event (GtkWidget *widget,
 }
 
 
+// libconic connection event
 static void
 ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
   ConIcConnectionEvent *event, gpointer user_data)
@@ -1031,7 +1074,7 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 
 	ConIcConnectionStatus status = con_ic_connection_event_get_status(event);
 	const gchar *iap_id = con_ic_event_get_iap_id(CON_IC_EVENT(event));
-	const gchar *bearer_id = con_ic_event_get_bearer_type(CON_IC_EVENT(event));
+	const gchar *bearer_type = con_ic_event_get_bearer_type(CON_IC_EVENT(event));
 
 	// Problem: When switching from GPRS to/from WLAN
 	// we receive STATUS_CONNECTED for the new IAP first,
@@ -1053,6 +1096,7 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 				self->disconnecting_iap = NULL;
 			}
 
+			// in disconnected state, connected_bearer should be NULL
 			if (self->connected_bearer) {
 				g_free(self->connected_bearer);
 				self->connected_bearer = NULL;
@@ -1070,7 +1114,7 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 
 		case CON_IC_STATUS_CONNECTED:
 
-			g_debug(WIDGET_NAME": connection event: connected bearer: %s", bearer_id);
+			g_debug(WIDGET_NAME": connection event: connected bearer: %s", bearer_type);
 
 			// we are online now
 
@@ -1080,8 +1124,8 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 			}
 
 			// save connected bearer
-			if (bearer_id)
-				self->connected_bearer = g_strdup(bearer_id);
+			if (bearer_type)
+				self->connected_bearer = g_strdup(bearer_type);
 
 			// ignore online event if bearer type string does not begin
 			// with "WLAN_" and conn_wlan_only option is set
@@ -1143,6 +1187,7 @@ ccchh_dooris_desktop_widget_connection_event(ConIcConnection *connection,
 }
 
 
+// configuration dialog response event - save or discard changes
 static void 
 ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
   gint response_id, gpointer user_data)
@@ -1155,6 +1200,8 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 
 	// only use new settings if user pressed the OK button
 	switch (response_id) {
+
+		// this is the "Defaults" button response
 		case GTK_RESPONSE_NO:
 			g_debug(WIDGET_NAME": restoring default configuration");
 
@@ -1176,6 +1223,7 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 
 			break;
 
+		// apply and save changes
 		case GTK_RESPONSE_ACCEPT:
 			g_debug(WIDGET_NAME": response accept");
 
@@ -1218,7 +1266,7 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 				g_debug(WIDGET_NAME": notification disabled");
 			}
 
-			// monitor internet connection state, only update if online
+			// monitor internet connection state, update only if online
 			self->use_connection = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->connection_button));
 
 			if (self->use_connection) {
@@ -1248,15 +1296,16 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 			// use only WLAN connections
 			self->conn_wlan_only = hildon_check_button_get_active(HILDON_CHECK_BUTTON(self->connwlan_button));
 			if (self->conn_wlan_only) {
-				g_debug(WIDGET_NAME": use only WLAN network");
+				g_debug(WIDGET_NAME": use only WLAN networks");
 			} else {
 				g_debug(WIDGET_NAME": use any network");
 			}
 
 			save_settings(self);
 
-			// fall through to dispose dialog window
+			// fall through to dialog window disposal
 
+		// if the user cancelled the dialog, do nothing
 		default:
 			g_debug(WIDGET_NAME": response ID was %i", response_id);
 
@@ -1293,7 +1342,7 @@ update_picker_changed_value_cb (HildonPickerButton *picker, gpointer data)
 }
 
 
-// use_connection button callback - set force_connection (in-)sensitive
+// use_connection button callback - set dependent buttons (in-)sensitive
 static void
 connection_button_toggled_cb (HildonCheckButton *button, gpointer data)
 {
@@ -1305,6 +1354,7 @@ connection_button_toggled_cb (HildonCheckButton *button, gpointer data)
 }
 
 
+// open configuration dialog
 static void 
 ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED, 
   gpointer user_data G_GNUC_UNUSED)
@@ -1330,16 +1380,23 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 			self);
 
 		// set dialog size
-		// TODO: is this the right way?
-		gtk_window_set_default_size (GTK_WINDOW(self->settings_window), 410, 400);
+		// TODO: is this the right way to do in Hildon? which w/h are best?
+		gtk_window_set_default_size(GTK_WINDOW(self->settings_window), 400, 400);
 
 		// arrange the inputs
 		// we put controls and labels into hbox-es
 		// and stack them into a vbox
 
 		GtkWidget *vbox = gtk_vbox_new (FALSE, HILDON_MARGIN_DEFAULT);
-//		GtkWidget *vbox = GTK_DIALOG(self->settings_window)->vbox;
-//		gtk_container_set_border_width(GTK_CONTAINER (vbox), HILDON_MARGIN_DEFAULT);
+
+		// pack the filled vbox into a scrollable area
+		GtkWidget *pan = hildon_pannable_area_new ();
+		hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA (pan), vbox);
+
+		// add the area into the window
+		gtk_box_pack_start (GTK_BOX(GTK_DIALOG(self->settings_window)->vbox), GTK_WIDGET(pan), TRUE, TRUE, 0);
+
+		// define the controls
 
 		GtkWidget *hbox = gtk_hbox_new (FALSE, HILDON_MARGIN_DEFAULT);
 		gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
@@ -1453,15 +1510,6 @@ ccchh_dooris_desktop_widget_on_show_settings (GtkWidget *widget G_GNUC_UNUSED,
 		gtk_button_set_alignment(GTK_BUTTON(self->connwlan_button), 0, 0.5);
 		gtk_box_pack_start (GTK_BOX (hbox), self->connwlan_button, TRUE, TRUE, 0);
 
-		// pannable area provides scrolling
-		GtkWidget *pan = hildon_pannable_area_new ();
-
-		/* pack the filled vbox into the scrolled window */
-		hildon_pannable_area_add_with_viewport (HILDON_PANNABLE_AREA (pan), vbox);
-
-		/* Add the area into the window*/
-		gtk_box_pack_start (GTK_BOX(GTK_DIALOG(self->settings_window)->vbox), GTK_WIDGET(pan), TRUE, TRUE, 0);
-
 		g_object_unref(sizegroup);
 	}
 
