@@ -49,6 +49,9 @@
 #define IMAGE_COLOR_HIGHLIGHT	1, 1, 1
 #define IMAGE_ALPHA_HIGHLIGHT	0.65
 
+#define IMAGE_COLOR_DIM			0.15, 0.15, 0.15
+#define IMAGE_ALPHA_DIM			0.47
+
 #define IMAGE_ALPHA_NORMAL		1.0
 #define IMAGE_ALPHA_PROGRESS	0.65
 
